#include <cmath>
#include <cstdlib>
#include <format>
#include<glad/glad.h>
#include<GLFW/glfw3.h>
#include <chrono>
#include <iostream>
#include <ostream>
#include <stdexcept>
#include <thread>

const float POINT_SIZE = 32.0f;

class BouncyBall{
public:
    int bx=0,by=0,bsx=5,bsy=7;
    BouncyBall(int x,int y,int speedX,int speedY):bx(x),by(y),bsx(speedX),bsy(speedY){}
    BouncyBall* update(){
        bx+=bsx,by+=bsy;
        if(bx>=1000||bx<=-1000){
            bsx=-bsx;
        }
        if(by>=1000||by<=-1000){
            bsy=-bsy;
        }
        return this;
    }
};

void render(){
    glBegin(GL_TRIANGLES);
    glColor3f(1.0f,0.0f,0.0f);
    glVertex3f(0.0f,0.5f,0.0f);
    glColor3f(0.0f,1.0f,0.0f);
    glVertex3f(-0.5,-0.5,0.0f);
    glColor3f(0.0f,0.0f,1.0f);
    glVertex3f(0.5f,-0.5f,0.0f);
    glEnd();

    glBegin(GL_POINTS);
    static float num = 0.0f;
    static float speed = 0.01f;
    num+=speed;
    if(num>1.0f){
        num=1.0f;
        speed = -0.01f;
    }else if(num<0.0f){
        num=0.0f;
        speed=0.01f;
    }
    glColor3f(num/1.11f,1.0f-(num/1.12f),num/1.13f);
    
    static BouncyBall b1(0,0,5,6),b2(500,70,3,-6),b3(-250,-100,-40,-25);
    glVertex3f(float(b1.bx)/1000.0f,float(b1.by)/1000.0f,0.0f);
    glVertex3f(float(b2.bx)/1000.0f,float(b2.by)/1000.0f,0.0f);
    glVertex3f(float(b3.bx)/1000.0f,float(b3.by)/1000.0f,0.0f);
    b1.update(),b2.update(),b3.update();
    glEnd();
}

int main(){
    // 初始化GLFW
    if(glfwInit()==0){
        throw std::runtime_error("failed to init GLFW");
    }
    GLFWwindow *window = glfwCreateWindow(640,480,"Example",nullptr,nullptr);

    if(window==nullptr){
        glfwTerminate();
        throw std::runtime_error("GLFW failed to create window");
    }

    glfwMakeContextCurrent(window);//创建上下文

    if(gladLoadGL()==0){  //加载GL函数
        glfwTerminate();
        throw std::runtime_error("GLAD Load GL functions failed");
    }
    
    std::cout<<"OpenGL version:"<<glGetString(GL_VERSION)<<"\n";

    glEnable(GL_POINT_SMOOTH);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
    glPointSize(POINT_SIZE);

    while(glfwWindowShouldClose(window)==false){
        glClear(GL_COLOR_BUFFER_BIT);
        render();
        glfwSwapBuffers(window);
        glfwPollEvents();
        std::this_thread::sleep_for(std::chrono::milliseconds(1000/120));
    }
    return 0;
}

#ifdef _MSC_VER 
#include<Windows.h>
int WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow) {
    main();
    return 0;
}
#endif